; HDD boot loader for QNX 1.2
; by Mihai Gaitos, mihaig@hawk.ro, 2021-01
; assemble using fasm boot.asm
; see crt_trk, crt_head, crt_sec below for kernel location
; sec_to_read is kernel size in sectors
format binary
use16
org 0
	cli
	cld
	mov ax,0x2000
	mov es,ax
	mov ax,0x07c0
	mov ds,ax
	mov ax,0x0
	mov si,ax
	mov di,ax
	mov cx,0x200
	rep movsb
	jmp 0x2000:$+5

	mov ax,cs
	mov ds,ax
	mov ss,ax

	mov ax,0x800
	mov sp,ax
	
	sti

	mov ax,first_msg
	call print_msg
	mov ax,60h
	mov es,ax
	dec [sec_to_read]

@@boot_read:
	mov ax,[crt_trk]
	mov cl,6
	shl ah,cl
	mov cl,ah
	mov ch,al
					; ch contains low order bits of track
					; cl contains 2 highest bits of track
					; in the uppper-most 2 bits 
	or cl,[crt_sec]	; and sector in lower 6 bits
	mov dh,[crt_head]
	mov dl,0x80
	xor bx,bx
	mov ax,201h		; ah = 2 (read sectors), al = 1 (count)
	int 13h
	jnc @@next
		mov ax,err_msg
		call print_msg
	@@next:
	mov al,'.'
	call print_char
;	mov ax,[crt_trk]
;	call print_hex16
;	mov al,','
;	call print_char
;	mov al,[crt_head]
;	call print_hex8
;	mov al,','
;	call print_char
;	mov al,[crt_sec]
;	call print_hex8
;	mov ax,msg_ok
;	call print_msg
	
	mov ax,es
	add ax,20h
	mov es,ax

	inc [crt_sec]
	cmp [crt_sec],18 ; change to SecPerTrk+1
	jb @f
	mov [crt_sec],1
	inc [crt_head]
	cmp [crt_head],4 ; change to HeadsPerTrk
	jb @f
	mov [crt_head],0
	inc [crt_trk]
	@@:
	dec [sec_to_read]
jnz @@boot_read

; ********************
; * LOADING FINISHED *
; ********************
mov ax,msg_done
call print_msg

mov al,0xff
out 0x21,al
cli
mov ax,0x0070
mov ds,ax
mov es,ax
mov ss,ax
jmp 0x0070:0x0080

print_char:
	mov ah,0x0e
	xor bx,bx
	int 0x10
	ret

print_msg:
	mov si,ax
	@@:
		lodsb
		cmp al,0
		jz @f
		call print_char
	jmp @b
	@@:
ret

print_hdigit:
	and ax,0x000f
	mov bx,hexfigs
	add bx,ax
	mov al,[bx]
	call print_char
	ret

print_hex16:
	push ax
	mov cl,12
	shr ax,cl
	call print_hdigit
	pop ax

	push ax
	mov cl,8
	shr ax,cl
	call print_hdigit
	pop ax

	push ax
	mov cl,4
	shr ax,cl
	call print_hdigit
	pop ax

	call print_hdigit
ret

print_hex8:
	push ax
	mov cl,4
	shr al,cl
	call print_hdigit
	pop ax
	call print_hdigit
	ret


; data
	sec_to_read	dw	202
	crt_trk		dw	301
	crt_head	db	0
	crt_sec		db	1
	first_msg	db	"Loading",10,13,0
	msg_ok		db	" ok",10,13,0
	err_msg		db	"Error reading sector!",0
	msg_done	db	10,13,"Done.",10,13,0
	hexfigs		db	"0123456789ABCDEF"

